<?php

namespace App\Exports;

use App\Models\DivisionModel;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use App\Models\VehicleTrakingReportModel;
use App\Models\VehicleModel;

class RouteDeviationExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['division_id']     = session::has('division_id') ? session::get('division_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));
        $aData['records']      = VehicleTrakingReportModel::get_route_deviation($aData);
        $default_points = 0;
        $cover_points = 0;
        $default_stop_points = 0;
        $actual_stop_points = 0;
        $today_net_weight = 0;
        $total_fine = 0;
        
        if( $aData['records']){
            foreach($aData['records'] as $key=>$aValue){ 
                $default_points += isset($aValue->default_points) ? $aValue->default_points : 0;
                $cover_points += isset($aValue->cover_points) ? $aValue->cover_points : 0;
                $default_stop_points += isset($aValue->default_stop_points) ? $aValue->default_stop_points : 0;
                $actual_stop_points += isset($aValue->actual_stop_points) ? $aValue->actual_stop_points : 0;
                $today_net_weight += $aValue->today_net_weight;
                
                if(!empty($aValue->percentage)){
                    $percentage = ceil($aValue->percentage);
                    $percentage_deviation = (100 - $percentage);
                }else{
                    $percentage           = '0';
                    $percentage_deviation = '0';
                }
                
                if(!empty($aValue->stoppage_percentage)){
                    $stoppage_percentage = ceil($aValue->stoppage_percentage);
                }else{
                    $stoppage_percentage = '0'; 
                }
                
                $total_fine += $aValue->deviation_fine;
                $aData['excel'][] = array(
                    'vehicle_number_name' => $aValue->vehicle_number_name,
                    'vehicle_type' => $aValue->vehicle_type ,
                    'route_name' => $aValue->route_name ,
                    'default_points' => $aValue->default_points ? ceil($aValue->default_points) : "0" ,
                    'cover_points' => $aValue->cover_points ? ceil($aValue->cover_points) : "0" ,
                    'percentage' => $percentage ? $percentage.'%' : "0%",
                    'percentage_deviation' => $percentage_deviation ? $percentage_deviation.'%' : "0%",
                    'default_stop_points' => $aValue->default_stop_points ? ceil($aValue->default_stop_points) : "0",
                    'actual_stop_points' => $aValue->actual_stop_points ? ceil( $aValue->actual_stop_points) : "0",
                    'stoppage_percentage' => $stoppage_percentage ? $stoppage_percentage : "0",
                    'today_net_weight' => $aValue->today_net_weight ? ceil($aValue->today_net_weight) : "0",
                    'deviation_fine' => $aValue->deviation_fine ? $aValue->deviation_fine : "0"
                ); 
            }
        }else{
            $aData['excel'][] = array(
            );     
        }

        return $aData['excel'];
    }

    public function headings(): array
    {   
        $aData['division_id']     = session::has('division_id') ? session::get('division_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));

        $division_name = "-";
        if($aData['division_id'] != 0){
            $div = DivisionModel::where('id', $aData['division_id'])->get('division_name');
            foreach($div as $val){
                $division_name = $val->division_name;
            }
        }

        $vehicle_types = "-";
        if($aData['vehicle_type_id'] != 0){
            $vehicle = DB::table('vehicle_types')->where('id',  $aData['vehicle_type_id'])->get('type'); 
            foreach($vehicle as $val){
                    $vehicle_types = $val->type;
            }
        }
               
        return [
                ['Report Name: Route Deviation Report.'],
                ['Division Name : '. $division_name ],
                //['Vehicle Type : '.$vehicle_types ],
                ['Date : '. $aData['date']],
                [],
                [ 
                    'Vehicle Number',
                    'Vehicle Type',
                    'Route Name',                   
                    'Default Route Point',                   
                    'Covered Route Point',                   
                    'Route Coverage Percentage',  
                    'Route Deviation Percentage',                   
                    'Dafault Collection Stoppage/Gate Points',                   
                    'Covered Stoppage/Gate Points',                   
                    'Covered Stoppage/Gate Percentage',                  
                    'Today`s Weight',                   
                    'Fine '                
                ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');

                $event->sheet->mergeCells('A1:C1');
                $event->sheet->mergeCells('A2:C2');
                $event->sheet->mergeCells('A3:C3');
                $event->sheet->mergeCells('A4:C4');
               
            
                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                
                $event->sheet->getStyle('A1:Z5')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
            },
        ];
    }
  
}

