<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use App\Models\ReportModel;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use App\Models\VehicleTrakingReportModel;


class VehicleStoppageExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['iOffset'] = 0;
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['time_interval']   = session::has('time_interval') ? session::get('time_interval') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));
        
        $aData['records'] = VehicleStoppageExport::get_vehicle_stoppages($aData['vehicle_id'],$aData['date'],$aData['time_interval'],$aData['iOffset']);
        if( $aData['records']){
            foreach($aData['records'] as $aValue){ 
                $aData['excel'][] = array(
                    'date' => $aValue->record_date,
                    'vehicle_number' =>  $aValue->vehicle_name->vehicle_number,
                    'address' => (!empty($aValue->address)) ? $aValue->address : '-' ,
                    'record_time' =>  $aValue->record_time,
                    'start_time' =>  $aValue->start_time ? date('H:i:s',$aValue->start_time) : "-",
                    'stopped_period' =>  $aValue->stopped_period ? gmdate('H:i:s',$aValue->stopped_period) : "-"
                ); 
            }

        }else{
            $aData['excel'][] = array(
            );     
        }
        
        return $aData['excel'];
    }

    public function get_vehicle_stoppages($vehicle_master_id,$date,$time_interval,$offset = 0){
           $aData['records'] = array();
           if(!empty($vehicle_master_id)){
            $route_vehicles = VehicleTrakingReportModel::get_route_vehicles($vehicle_master_id,$date);
            $vehicle_stopped_folder = public_path('vehicle_stopped');
            $date1 = date('Ymd',strtotime($date));
            foreach($route_vehicles as $aVeh){
                $vehicle_master_id = $aVeh;
                $vehicle_name = VehicleTrakingReportModel::get_vehicle_name($vehicle_master_id);
                $filepath = $vehicle_stopped_folder.'/'.$date1.'/'.$vehicle_master_id.'.json';
                $iOffset = $offset;
                if(file_exists($filepath)){
                    $handle = fopen($filepath, "r");
                    $route = fgets($handle);
                    fclose($handle);
                    $route = trim($route);
                    $firstchar = substr($route,0,1);
                    if($firstchar != '['){
                        $route = '['.$route.']';
                    }
                    $aRoute = json_decode($route);
                    if(!empty($aRoute)){
                        if(!empty($time_interval)){
                                $aTime_intervals = config('custom.aTime_intervals');
                                $aStopped_limit = $aTime_intervals[$time_interval];
                                $last = 0;
                                if($aStopped_limit['last'] != 1){
                                    $min_seconds = $aStopped_limit['min'] * 60;
                                    $max_seconds = $aStopped_limit['max'] * 60;
                                }else{
                                    $last =1;
                                    $max_seconds = $aStopped_limit['min'] * 60;
                                }
                        }					
                        for($i=0;$i<count($aRoute);$i++){
                            if(array_key_exists($iOffset,$aRoute)){
                                $aRoute[$iOffset]->vehicle_name = $vehicle_name;
                                if(!empty($aRoute[$iOffset]->start_time)){
                                    if(!empty($time_interval)){
                                        if(!empty($last)){
                                            if($aRoute[$iOffset]->stopped_period > $max_seconds){
                                                $aData['records'][] = $aRoute[$iOffset];
                                            }
                                        }else{
                                            if($aRoute[$iOffset]->stopped_period >$min_seconds && $aRoute[$iOffset]->stopped_period <=$max_seconds){
                                                $aData['records'][] = $aRoute[$iOffset];
                                            }
                                        }
                                    }else{
                                        $aData['records'][] = $aRoute[$iOffset];
                                    }
                                }
                            }
                            $iOffset++;
                        }
                    } 
                }
            }
          }
          return $aData['records'];
        } 

    public function headings(): array
    {   
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['time_interval']   = session::has('time_interval') ? session::get('time_interval') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));
        $result = DB::table('vehicle_master')->where("id", $aData['vehicle_id'])->get("vehicle_number");
        
        if($result){
            foreach($result as $val){
                $vehicle_number = $val->vehicle_number;
            }
        }else{
            $vehicle_number = "-";
        }

        $aTime_intervals = config('custom.aTime_intervals');
        foreach($aTime_intervals as $key=>$val){
            if($key == $aData['time_interval'] ){
                $time_interval = $val['text'];
                break;
            }
           else{
                $time_interval = "-";
            }
            
        }

        return [
                ['Report Name: Vehicle Stoppage Report.'],
                ['Vehicle : '. $vehicle_number  ],
                ['Date : '.  $aData['date'] ],
                ['Time Interval : '. $time_interval ],
                [],
                [ 
                    'Date ',
                    'Vehicle Number ',
                    'Location  ',
                    'Start Time ',
                    'Stop Time ' ,
                    'Stoppage Period '
                ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');

                $event->sheet->mergeCells('A1:C1');
                $event->sheet->mergeCells('A2:C2');
                $event->sheet->mergeCells('A3:C3');
                $event->sheet->mergeCells('A4:C4');
                $event->sheet->mergeCells('A5:Z5');

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('6')->setRowHeight(25);

                $event->sheet->getStyle('A1:Z6')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
            },
        ];
    }
  
}

