<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use App\Models\AdminModel;
use App\Models\Module;
use App\Models\AdminUserRight;
use App\Models\PakageModel;
use Illuminate\Support\Facades\DB;


class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */


    public function __construct()
    {
        $module_link = request()->segments(0);
        if (isset($module_link[0])) {
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next) {
                $this->mysecurity->check_access(Session::get('logged_in')['id'], $this->module_id, 1);

                return $next($request);
            });
        }
    }

    public function clear_search(){
        session::forget('user_name');
        return redirect('/users');
    }

    public function index(Request $request)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
            session(['user_name' => $request->user_name]);
         }
         
        $aData['user_name'] = session::has('user_name') ? session::get('user_name') : "";

        if(!empty($aData['user_name'])){ 
            $aData['search'] = 1;
        }

        $aData['aUsers'] = AdminModel::get_all_users($aData)->paginate(30);  
        return view('users.list', $aData);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        return view('users.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'firstname' => 'required',
            'lastname' => 'required',
            'email' => 'email:rfc,dns',
            'password' => 'confirmed|min:5'
        ]);

        $result = AdminModel:: where('deleted', 0)->get('email');
        $data = array();
        foreach ($result as $val){
            $data[] = $val->email;
        }
        if(in_array($request->email,$data)){
            return redirect()->back()->with('error', 'Email Already Exist.');
        }
        $users = new AdminModel();
        $users->emp_id = 0;
        $users->firstname = $request->firstname;
        $users->lastname = $request->lastname;
        $users->email = $request->email;
        $users->password = md5($request->password);
        $users->gender = $request->gender;
        $users->active = 1;
        $users->created = strtotime('now');
        $users->created_by = 0;
        $users->updated_date = 0; 
        $users->save();
        return redirect()->route('users.index')->with('success', 'User Added Successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
       
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array();
        $aData['aUser'] = AdminModel::find($id);

        // Module Access
        $aData['aModules'] = Module::all()->where('active','=',1);
        $aAccess = AdminUserRight::where('user_id', $id)->get(); 
        foreach($aAccess as $val){            
            $aData['aAccess'][$val->module_id] = $val->access;
        }

        //Show Pakage of user
        $aData['Pakage_name'] = DB::select('SELECT up.*,(SELECT Pakage_name FROM admin_pakages WHERE id = up.Pakage_id) as Pakage_name FROM user_Pakages up WHERE up.user_id='.$id);

        $aData['aPakages'] = PakageModel::whereNotIn('id', DB::table('user_Pakages')->where('user_id', '=', $id)
                                            ->pluck('Pakage_id'))
                                            ->get();
        
        return view('users.edit', $aData);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->validate($request,[
            'firstname' => 'required',
            'lastname' => 'required',
            'email' => 'required|email:rfc,dns',
            'gender' => 'required'
        ]);

        if(!empty($request->email)){
            $res = AdminModel::where('id', $id)->get("email");
            $email = "";
            foreach ($res as $val){
                $email = $val->email;
            }
        }if(!empty($request->email)){
            $res = AdminModel::where('id', $id)->get("email");
            $email = "";
            foreach ($res as $val){
                $email = $val->email;
            }
            if($email == $request->email){
                $user = AdminModel::findOrFail($id);
                $user->emp_id = 0;
                $user->firstname = $request->firstname;
                $user->lastname = $request->lastname;
                $user->email = $request->email;
                if($request->password){
                    $user->password = md5($request->password);
                }
                $user->gender = $request->gender;
                $user->updated_date =  strtotime('now'); 
                $user->save();
        
                return redirect()->route('users.index')->with('success', 'User Updated Successfully');
            }else{
                $result = AdminModel:: where('email', '!=' , $email)->get('email');
                $data = array();
                foreach ($result as $val){
                    $data[] = $val->email;
                }
                if(in_array($request->email,$data)){
                    return redirect()->back()->with('error', 'Email Already Exist.');
                }else{
                    $user = AdminModel::findOrFail($id);
                    $user->emp_id = 0;
                    $user->firstname = $request->firstname;
                    $user->lastname = $request->lastname;
                    $user->email = $request->email;
                    if($request->password){
                        $user->password = md5($request->password);
                    }
                    $user->gender = $request->gender;
                    $user->updated_date =  strtotime('now'); 
                    $user->save();
                    return redirect()->route('users.index')->with('success', 'User Updated Successfully');                }
            }
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        AdminModel::where('id',$id)->update(['deleted'=>1]);
        return back()->with('error', 'Users deleted successfully');

    }

    public function change_active_status(Request $request){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $user = AdminModel::find($request->id);
        $user->active= $request->status;
        $user->save();
        return response()->json(['success'=>'User status change successfully.']);
    
    }

    public function module_access(Request $request){
        $user_id = $request->user_id;
        $access= $request->access;

        foreach($access as $key=>$val){
            $module_id=$key;
            $acs=$val;

            $data = AdminUserRight::where('user_id', $user_id)->where('module_id', $module_id)->get();

            if(!$data->isEmpty()){
                $updates= AdminUserRight::where('user_id', $user_id)->where('module_id', $module_id)->update(['access'=>$acs, 'updated_date'=>strtotime('now')]);
             
            }else{
             AdminUserRight::insert(['user_id'=>$user_id,'module_id'=>$module_id,'access'=>$acs, 'updated_date'=>strtotime('now')]);
            } 
        }
        return redirect()->route('users.index')->with('success', 'User Rights Updated Successfully'); 

    }

    public function user_Pakage(Request $request){
        
        DB::table('user_Pakages')->insert(['user_id'=>$request->user_id,'Pakage_id'=>$request->Pakage]);
        return back()->with('success', 'User Pakage added successfully');

    }

    public function remove_Pakage(Request $request){

        DB::table('user_Pakages')->where('user_id',$request->user_id)->where('Pakage_id',$request->Pakage_id)->delete();
        return back()->with('success', 'User Pakage deleted successfully');
    }

    public function show_Pakages(Request $request){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        $aData['Pakage_name'] = DB::select('SELECT up.*,(SELECT Pakage_name FROM admin_pakages WHERE id = up.Pakage_id) as Pakage_name FROM user_Pakages up WHERE up.user_id='.$request->user_id);
        $html = "";
        $i = 0;
        $html .= '<div><tr>
                    <th class= "text-center">Sr No.</th>
                    <th>Pakages</th>
                </tr>';
        if($aData['Pakage_name']){
            foreach($aData['Pakage_name'] as $val){
                $i++;
                $html .= '<tr>
                            <td class= "text-center">'.$i.'</td>
                            <td class="caption">'.$val->Pakage_name.'</td>
                        </tr>';
            }
        }else{
            $html .= '<tr>
            <td colspan="2" style="text-align:center; color:red;">No Record found</td>
            </tr>';
        }
        return $html;
    }
}
